import path from "node:path";
import pico from "picocolors";
import semver from "semver";
import fs from "fs-extra";
import { execa } from "execa";
const getPkgInfoList = async (targetDir) => {
  const pkgInfoList = [];
  const pkgDirList = fs.readdirSync(targetDir).filter((filename) => filename[0] !== ".");
  await Promise.all(
    pkgDirList.map(async (pkgDir) => {
      const pkgDirPath = path.join(targetDir, pkgDir);
      const pkgPath = path.join(pkgDirPath, "package.json");
      const pkgInfo = await fs.readJSON(pkgPath);
      pkgInfoList.push({
        pkgName: pkgInfo.name,
        pkgPath,
        pkgDirPath,
        pkgCurrentVersion: pkgInfo.version
      });
    })
  );
  return pkgInfoList;
};
const run = async (bin, args, opts = {}) => {
  return execa(bin, args, { stdio: "inherit", ...opts });
};
const step = (msg) => {
  return console.log(pico.cyan(msg));
};
const getVersionChoices = (pkgCurrentVersion) => {
  const currentBeta = pkgCurrentVersion.includes("beta");
  const currentAlpha = pkgCurrentVersion.includes("alpha");
  const isStable = !currentBeta && !currentAlpha;
  function inc(i, tag = currentAlpha ? "alpha" : "beta") {
    return semver.inc(pkgCurrentVersion, i, tag);
  }
  let versionChoices = [
    {
      title: "next",
      value: inc(isStable ? "patch" : "prerelease")
    }
  ];
  if (isStable) {
    versionChoices.push(
      {
        title: "beta-minor",
        value: inc("preminor")
      },
      {
        title: "beta-major",
        value: inc("premajor")
      },
      {
        title: "alpha-minor",
        value: inc("preminor", "alpha")
      },
      {
        title: "alpha-major",
        value: inc("premajor", "alpha")
      },
      {
        title: "minor",
        value: inc("minor")
      },
      {
        title: "major",
        value: inc("major")
      }
    );
  } else if (currentAlpha) {
    versionChoices.push({
      title: "beta",
      value: inc("patch") + "-beta.0"
    });
  } else {
    versionChoices.push({
      title: "stable",
      value: inc("patch")
    });
  }
  versionChoices.push({ value: "custom", title: "custom" });
  versionChoices = versionChoices.map((i) => {
    i.title = `${i.title} (${i.value})`;
    return i;
  });
  return versionChoices;
};
const updateVersion = async (pkgPath, version) => {
  const pkg = await fs.readJSON(pkgPath);
  pkg.version = version;
  fs.writeFileSync(pkgPath, JSON.stringify(pkg, null, 2) + "\n");
};
const publishPkg = async (pkdDir, tag) => {
  const publicArgs = ["publish", "--access", "public"];
  if (tag) {
    publicArgs.push(`--tag`, tag);
  }
  publicArgs.push(`--no-git-checks`);
  await run("pnpm", publicArgs, {
    cwd: pkdDir
  });
};
const getLatestTag = async (pkgName) => {
  const tags = (await run("git", ["tag"], { stdio: "pipe" })).stdout.split(/\n/).filter(Boolean);
  return tags.filter((tag) => tag.startsWith(`${pkgName}@`)).sort().reverse()[0];
};
const logRecentCommits = async (pkgName) => {
  const tag = await getLatestTag(pkgName);
  if (!tag)
    return;
  const sha = await run("git", ["rev-list", "-n", "1", tag], {
    stdio: "pipe"
  }).then((res) => res.stdout.trim());
  console.log(
    pico.bold(
      `
${pico.blue(`i`)} Commits of ${pico.green(pkgName)} since ${pico.green(
        tag
      )} ${pico.gray(`(${sha.slice(0, 5)})`)}`
    )
  );
  await run(
    "git",
    [
      "--no-pager",
      "log",
      `${sha}..HEAD`,
      "--oneline",
      "--",
      `packages/${pkgName}`
    ],
    { stdio: "inherit" }
  );
  console.log();
};
export {
  getLatestTag,
  getPkgInfoList,
  getVersionChoices,
  logRecentCommits,
  publishPkg,
  run,
  step,
  updateVersion
};
